@@ -1,10 +1,11 @@
 """pyvelop CLI."""
 
 # region #-- imports --#
+import json
 import logging
 import re
 import uuid
-from typing import Dict, List, Tuple
+from typing import Dict, List, Set, Tuple
 
 import aiohttp
 import asyncclick as click
@@ -119,6 +120,18 @@ class StandardCommand(click.Command):
             self.params.insert(0, opt)
 
 
+MESH_ALLOWED_ACTIONS: Set = (
+    "channel_scan_info",
+    "channel_scan_start",
+    "guest_wifi_off",
+    "guest_wifi_on",
+    "parental_control_off",
+    "parental_control_on",
+    "speedtest_results",
+    "speedtest_start",
+    "speedtest_state",
+    "update_check_start",
+)
 DEF_INDENT: int = 2
 
 click.anyio_backend = "aysncio"
@@ -247,6 +260,7 @@ async def mesh_details(
                         ),
                         ("parental_control_enabled", "Parental Control Enabled"),
                         ("wps_state", "WPS Enabled"),
+                        ("is_channel_scan_running", "Channel Scan Running"),
                         ("homekit_enabled", "HomeKit Integration Enabled"),
                         ("homekit_paired", "HomeKit Integration Paired"),
                         ("client_steering_enabled", "Client Steering Enabled"),
@@ -308,6 +322,41 @@ async def mesh_details(
             )
 
 
+@mesh_group.command(cls=StandardCommand, name="action")
+@click.argument("action", type=click.Choice(MESH_ALLOWED_ACTIONS, case_sensitive=False))
+@click.pass_context
+async def mesh_action(
+    ctx: click.Context,
+    action: str,
+    **_,
+) -> None:
+    """Carry out a specified action on the mesh."""
+    if mesh_obj := await mesh_connect(ctx):
+        async with mesh_obj:
+            if action == "channel_scan_info":
+                ret = await mesh_obj.async_get_channel_scan_info()
+            elif action == "channel_scan_start":
+                ret = await mesh_obj.async_start_channel_scan()
+            elif action == "guest_wifi_off":
+                ret = await mesh_obj.async_set_guest_wifi_state(state=False)
+            elif action == "guest_wifi_on":
+                ret = await mesh_obj.async_set_guest_wifi_state(state=True)
+            elif action == "parental_control_off":
+                ret = await mesh_obj.async_set_parental_control_state(state=False)
+            elif action == "parental_control_on":
+                ret = await mesh_obj.async_set_parental_control_state(state=True)
+            elif action == "speedtest_results":
+                ret = await mesh_obj.async_get_speedtest_results()
+            elif action == "speedtest_start":
+                ret = await mesh_obj.async_start_speedtest()
+            elif action == "speedtest_state":
+                ret = await mesh_obj.async_get_speedtest_state()
+            elif action == "update_check_start":
+                ret = await mesh_obj.async_check_for_updates()
+
+    print(json.dumps(ret))
+
+
 @cli.group(name="node")
 @click.help_option()
 async def node_group() -> None:
