@@ -7,6 +7,9 @@ import asyncio
 import base64
 import json
 import logging
+from collections import defaultdict
+from dataclasses import dataclass
+from enum import Enum
 from typing import Any, Dict, List, Optional
 
 import aiohttp
@@ -39,7 +42,7 @@ def jnap_url(target) -> str:
     return f"http://{target}/JNAP/"
 
 
-class Actions:
+class Actions(str, Enum):
     """Represents the available actions."""
 
     ROOT: str = "http://linksys.com/jnap"
@@ -47,6 +50,7 @@ class Actions:
     CHECK_PASSWORD: str = f"{ROOT}/core/CheckAdminPassword"
     DELETE_DEVICE: str = f"{ROOT}/devicelist/DeleteDevice"
     GET_BACKHAUL: str = f"{ROOT}/nodes/diagnostics/GetBackhaulInfo"
+    GET_CHANNEL_SCAN_STATUS: str = f"{ROOT}/nodes/setup/GetSelectedChannels"
     GET_DEVICES: str = f"{ROOT}/devicelist/GetDevices3"
     GET_FIRMWARE_UPDATE_SETTINGS: str = (
         f"{ROOT}/firmwareupdate/GetFirmwareUpdateSettings"
@@ -78,20 +82,21 @@ class Actions:
     SET_PARENTAL_CONTROL_INFO: str = (
         f"{ROOT}/parentalcontrol/SetParentalControlSettings"
     )
+    START_CHANNEL_SCAN: str = f"{ROOT}/nodes/setup/StartAutoChannelSelection"
     START_SPEEDTEST: str = f"{ROOT}/healthcheck/RunHealthCheck"
     TRANSACTION: str = f"{ROOT}/core/Transaction"
     UPDATE_FIRMWARE: str = f"{ROOT}/nodes/firmwareupdate/UpdateFirmwareNow"
 
 
+@dataclass
 class Defaults:
     """Represents the default payloads required for requests."""
 
-    PAYLOADS: Dict[str, Dict] = {
-        Actions.GET_SPEEDTEST_RESULTS: {
-            "healthCheckModule": "SpeedTest",
-            "includeModuleResults": True,
-            "lastNumberOfResults": 1,
-        },
+    payloads = defaultdict(dict)
+    payloads[Actions.GET_SPEEDTEST_RESULTS] = {
+        "healthCheckModule": "SpeedTest",
+        "includeModuleResults": True,
+        "lastNumberOfResults": 1,
     }
 
 
@@ -128,9 +133,8 @@ class Request:
         self._session: Optional[
             aiohttp.ClientSession
         ] = session or aiohttp.ClientSession(raise_for_status=True)
-        self._target: str = target
 
-        self._jnap_url: str = jnap_url(target=self._target)
+        self._jnap_url: str = jnap_url(target=target)
 
     async def execute(self, timeout: int = 10) -> Response:
         """Send the request.
