@@ -7,7 +7,7 @@ import asyncio
 import json
 import logging
 import time
-from typing import Dict, List, Mapping, Optional, Tuple
+from typing import Any, Dict, List, Mapping, Optional, Tuple
 
 import aiohttp
 
@@ -24,7 +24,7 @@ from .exceptions import (
     MeshTooManyMatches,
 )
 from .logger import Logger
-from .node import NODE_TYPE_PRIMARY, Node
+from .node import Node, NodeType
 
 # endregion
 
@@ -33,6 +33,7 @@ _LOGGER_VERBOSE = logging.getLogger(f"{__name__}.verbose")
 
 # region #-- attributes for results --#
 ATTR_BACKHAUL_INFO: str = "backhaul"
+ATTR_CHANNEL_SCAN_INFO: str = "channel_scan_info"
 ATTR_FIRMWARE_UPDATE_SETTINGS: str = "firmware_update_settings"
 ATTR_GUEST_NETWORK_INFO: str = "guest_network"
 ATTR_HOMEKIT_SETTINGS: str = "homekit_settings"
@@ -54,6 +55,7 @@ ATTR_WPS_SERVER_SETTINGS: str = "wps_server_settings"
 
 JNAP_ACTION_TO_ATTRIBUTE: dict = {
     api.Actions.GET_BACKHAUL: ATTR_BACKHAUL_INFO,
+    api.Actions.GET_CHANNEL_SCAN_STATUS: ATTR_CHANNEL_SCAN_INFO,
     api.Actions.GET_DEVICES: ATTR_RAW_DEVICES,
     api.Actions.GET_FIRMWARE_UPDATE_SETTINGS: ATTR_FIRMWARE_UPDATE_SETTINGS,
     api.Actions.GET_GUEST_NETWORK_INFO: ATTR_GUEST_NETWORK_INFO,
@@ -72,18 +74,6 @@ JNAP_ACTION_TO_ATTRIBUTE: dict = {
 }
 
 
-def _build_action_payload(transaction: List) -> List:
-    """Set default action for payload if none supplied."""
-    return list(
-        map(
-            lambda p: (
-                (dict(**p, **{"request": {}}) if p.get("request") is None else p)
-            ),
-            transaction,
-        )
-    )
-
-
 def _process_speedtest_results(
     speedtest_results=None, only_latest: bool = False, only_completed: bool = False
 ) -> List:
@@ -157,7 +147,7 @@ class Mesh:
     If you need live information then call the corresponding method.
     """
 
-    def __init__(
+    def __init__(  # pylint: disable=too-many-arguments
         self,
         node: str,
         password: str,
@@ -173,8 +163,6 @@ class Mesh:
         :param session: session to use in for interacting with the Mesh
         :param username: username to use; default admin
         """
-        # super().__init__()
-
         self._log_formatter = Logger()
 
         _LOGGER.debug(self._log_formatter.format("entered"))
@@ -258,7 +246,7 @@ class Mesh:
             self.__create_session()
 
         req = api.Request(
-            action=action,
+            action=action if not isinstance(action, api.Actions) else action.value,
             password=self.__password,
             payload=payload,
             raise_on_error=raise_on_error,
@@ -278,6 +266,7 @@ class Mesh:
         """Work is done here to gather the necessary details for mesh.
 
         :param include_backhaul: True to include backhaul details
+        :param include_channel_scan: True to include details about the channel scan process
         :param include_devices: True to include devices
         :param include_firmware_update: True to include the current firmware update details (does not issue a check)
         :param include_firmware_update_settings: True to include the current settings for firmware updates
@@ -299,9 +288,9 @@ class Mesh:
         )
 
         ret = {}
-        payload_safe: List = []
 
         # region #-- prepare the safe request --#
+        payload_safe: List = []
         # -- get the devices --#
         if kwargs.get("include_devices"):
             payload_safe.append({"action": api.Actions.GET_DEVICES})
@@ -309,10 +298,14 @@ class Mesh:
         if kwargs.get("include_firmware_update_settings"):
             payload_safe.append({"action": api.Actions.GET_FIRMWARE_UPDATE_SETTINGS})
 
-        # -- get the backhaul info  --#
+        # -- get the backhaul info --#
         if kwargs.get("include_backhaul") or kwargs.get("include_devices"):
             payload_safe.append({"action": api.Actions.GET_BACKHAUL})
 
+        # -- get the channel scan info --#
+        if kwargs.get("include_channel_scan"):
+            payload_safe.append({"action": api.Actions.GET_CHANNEL_SCAN_STATUS})
+
         # -- get the update check details --#
         if kwargs.get("include_firmware_update"):
             payload_safe.append({"action": api.Actions.GET_UPDATE_FIRMWARE_STATE})
@@ -339,7 +332,7 @@ class Mesh:
                 {
                     "action": api.Actions.GET_SPEEDTEST_RESULTS,
                     "request": {
-                        **api.Defaults.PAYLOADS.get(api.Actions.GET_SPEEDTEST_RESULTS),
+                        **api.Defaults.payloads[api.Actions.GET_SPEEDTEST_RESULTS],
                         "lastNumberOfResults": 10,
                     },
                 }
@@ -361,7 +354,17 @@ class Mesh:
 
         request_safe = self._async_make_request(
             action=api.Actions.TRANSACTION,
-            payload=_build_action_payload(transaction=payload_safe),
+            payload=list(
+                map(
+                    lambda r: {
+                        "action": r.get("action").value,
+                        "request": r.get(
+                            "request", api.Defaults.payloads[r.get("action")]
+                        ),
+                    },
+                    payload_safe,
+                )
+            ),
             raise_on_error=False,
         )
         # endregion
@@ -390,16 +393,20 @@ class Mesh:
             request_unsafe.append(
                 self._async_make_request(
                     action=api.Actions.TRANSACTION,
-                    payload=_build_action_payload(
-                        transaction=[
-                            {
-                                "action": api.Actions.GET_STORAGE_SMB_SERVER,
-                            },
-                            {
-                                "action": api.Actions.GET_STORAGE_PARTITIONS,
-                            },
-                        ]
-                    ),
+                    payload=[
+                        {
+                            "action": api.Actions.GET_STORAGE_SMB_SERVER,
+                            "request": api.Defaults.payloads[
+                                api.Actions.GET_STORAGE_SMB_SERVER
+                            ],
+                        },
+                        {
+                            "action": api.Actions.GET_STORAGE_PARTITIONS,
+                            "request": api.Defaults.payloads[
+                                api.Actions.GET_STORAGE_PARTITIONS
+                            ],
+                        },
+                    ],
                     raise_on_error=False,
                 )
             )
@@ -423,7 +430,7 @@ class Mesh:
         for response in responses:
             req, resp = response
             if req.action == api.Actions.TRANSACTION:
-                for idx, action_response in enumerate(resp.data):
+                for idx, action_response in enumerate(resp.data or {}):
                     _set_raw_value(
                         action=req.payload[idx].get("action"), data=action_response
                     )
@@ -433,127 +440,123 @@ class Mesh:
         # endregion
 
         # region #-- handle devices --#
-        if ATTR_RAW_DEVICES in ret:
-            devices = []
-            # region #-- build the properties for the device types --#
-            for device in ret[ATTR_RAW_DEVICES].get("devices", []):
-                device["results_time"]: int = int(time.time())
-                if "nodeType" in device:
-                    # region #-- determine the backhaul information --#
-                    device_backhaul = [
-                        bi
-                        for bi in ret.get(ATTR_BACKHAUL_INFO, {}).get(
-                            "backhaulDevices", []
-                        )
-                        if bi.get("deviceUUID") == device.get("deviceID")
-                    ]
-                    device_backhaul = device_backhaul[0] if device_backhaul else {}
-                    # endregion
+        devices: List[Device | Node] = []
+        # region #-- build the properties for the device types --#
+        for device in ret.get(ATTR_RAW_DEVICES, {}).get("devices", []):
+            device["results_time"]: int = int(time.time())
+            if "nodeType" not in device:
+                devices.append(Device(**device))
+            else:
+                # region #-- determine the backhaul information --#
+                device_backhaul = [
+                    bi
+                    for bi in ret.get(ATTR_BACKHAUL_INFO, {}).get("backhaulDevices", [])
+                    if bi.get("deviceUUID") == device.get("deviceID")
+                ]
+                # endregion
 
-                    # region #-- calculate if there is a firmware update available --#
-                    node_firmware: List | dict = {}
-                    if ATTR_UPDATE_FIRMWARE_STATE in ret:
-                        firmware_status = ret[ATTR_UPDATE_FIRMWARE_STATE].get(
+                # region #-- calculate if there is a firmware update available --#
+                node_firmware: List | dict = {}
+                if ATTR_UPDATE_FIRMWARE_STATE in ret:
+                    node_firmware = [
+                        firmware_details
+                        for firmware_details in ret[ATTR_UPDATE_FIRMWARE_STATE].get(
                             "firmwareUpdateStatus", []
                         )
-                        node_firmware = [
-                            firmware_details
-                            for firmware_details in firmware_status
-                            if firmware_details.get("deviceUUID")
-                            == device.get("deviceID")
-                        ]
-                        node_firmware = node_firmware[0] if node_firmware else {}
-                    # endregion
-
-                    devices.append(
-                        Node(
-                            **device,
-                            **{
-                                "backhaul": device_backhaul,
-                                "updates": node_firmware,
-                            },
-                        )
-                    )
-                else:
-                    devices.append(Device(**device))
-            # endregion
-
-            # region #-- post processing devices and nodes --#
-            for node in devices:
-                if node.__class__.__name__.lower() == "node":
-                    # region #-- calculate the connected devices for nodes --#
-                    connected_devices: List = []
-                    parent_name: Optional[str] = None
-                    for device in devices:
-                        for adapter in device.network:
-                            if adapter.get("parent_id") == node.unique_id:
-                                connected_devices.append(
-                                    {
-                                        "name": device.name,
-                                        "ip": adapter.get("ip"),
-                                        "type": adapter.get("type"),
-                                        "guest_network": adapter.get("guest_network"),
-                                    }
-                                )
-                            if node.parent_ip and not parent_name:
-                                if node.parent_ip == adapter.get("ip"):
-                                    parent_name = device.name
-                    setattr(node, "_Node__parent_name", parent_name)
-                    setattr(node, "_Node__connected_devices", connected_devices)
-                    # endregion
-                elif node.__class__.__name__.lower() == "device":
-                    # region #-- calculate parent name for devices --#
-                    attrib_connections = getattr(node, "_attribs", {}).get(
-                        "connections", []
-                    )
-                    parent: Optional[str] = None
-                    for conn in attrib_connections:
-                        if conn.get("parentDeviceID", ""):
-                            try:
-                                parent = [
-                                    device.name
-                                    for device in devices
-                                    if device.unique_id == conn.get("parentDeviceID")
-                                ][0]
-                            except IndexError:
-                                pass
-                    setattr(node, "_Device__parent_name", parent)
-                    # endregion
-
-                    network_adapater_macs = [
-                        adapter.get("mac") for adapter in node.network
+                        if firmware_details.get("deviceUUID") == device.get("deviceID")
                     ]
-                    # region #-- get the parental control details --#
-                    pc_schedule: List = []
-                    for mac in network_adapater_macs:
-                        for rule in ret.get(ATTR_PARENTAL_CONTROL_INFO, {}).get(
-                            "rules", []
+                # endregion
+
+                devices.append(
+                    Node(
+                        **device,
+                        **{
+                            "backhaul": device_backhaul[0] if device_backhaul else {},
+                            "updates": node_firmware[0] if node_firmware else {},
+                        },
+                    )
+                )
+        # endregion
+
+        # region #-- post processing devices and nodes --#
+        for node in devices:
+            if isinstance(node, Node):
+                # region #-- calculate the connected devices for nodes --#
+                connected_devices: List = []
+                parent_name: Optional[str] = None
+                for device in devices:
+                    for adapter in device.network:
+                        if adapter.get("parent_id") == node.unique_id:
+                            connected_devices.append(
+                                {
+                                    "name": device.name,
+                                    "ip": adapter.get("ip"),
+                                    "type": adapter.get("type"),
+                                    "guest_network": adapter.get("guest_network"),
+                                }
+                            )
+                        if (
+                            node.parent_ip
+                            and not parent_name
+                            and node.parent_ip == adapter.get("ip")
                         ):
-                            if mac in rule.get("macAddresses", []):
-                                pc_schedule.append(rule)
-                                break
-                    getattr(node, "_attribs", {})["parental_controls"] = pc_schedule
-                    # endregion
+                            parent_name = device.name
+                setattr(node, "_Node__parent_name", parent_name)
+                setattr(node, "_Node__connected_devices", connected_devices)
+                # endregion
+            elif isinstance(node, Device):
+                # region #-- calculate parent name for devices --#
+                attrib_connections = getattr(node, "_attribs", {}).get(
+                    "connections", []
+                )
+                parent: Optional[str] = None
+                for conn in attrib_connections:
+                    if conn.get("parentDeviceID", ""):
+                        try:
+                            parent = [
+                                device.name
+                                for device in devices
+                                if device.unique_id == conn.get("parentDeviceID")
+                            ][0]
+                        except IndexError:
+                            pass
+                setattr(node, "_Device__parent_name", parent)
+                # endregion
 
-                    # region #-- get additional connection details --#
-                    if ATTR_NETWORK_CONNECTIONS in ret:
-                        for mac in network_adapater_macs:
-                            for conn_details in ret[ATTR_NETWORK_CONNECTIONS].get(
-                                "nodeWirelessConnections", []
-                            ):
-                                node_connections: List[Mapping] = conn_details.get(
-                                    "connections", {}
-                                )
-                                for connection in node_connections:
-                                    if mac == connection.get("macAddress"):
-                                        getattr(node, "_attribs", {})[
-                                            "connection_details"
-                                        ] = connection
-                                        break
-                    # endregion
-            # endregion
+                network_adapater_macs = [adapter.get("mac") for adapter in node.network]
+                # region #-- get the parental control details --#
+                pc_schedule: List = []
+                for mac in network_adapater_macs:
+                    for rule in ret.get(ATTR_PARENTAL_CONTROL_INFO, {}).get(
+                        "rules", []
+                    ):
+                        if mac in rule.get("macAddresses", []):
+                            pc_schedule.append(rule)
+                            break
+                getattr(node, "_attribs", {})["parental_controls"] = pc_schedule
+                # endregion
 
-            ret[ATTR_PROCESSED_DEVICES] = devices or []
+                # region #-- get additional connection details --#
+                if (
+                    network_connections := ret.get(ATTR_NETWORK_CONNECTIONS)
+                ) is not None:
+                    for mac in network_adapater_macs:
+                        for conn_details in network_connections.get(
+                            "nodeWirelessConnections", []
+                        ):
+                            node_connections: List[Mapping] = conn_details.get(
+                                "connections", {}
+                            )
+                            for connection in node_connections:
+                                if mac == connection.get("macAddress"):
+                                    getattr(node, "_attribs", {})[
+                                        "connection_details"
+                                    ] = connection
+                                    break
+                # endregion
+        # endregion
+
+        ret[ATTR_PROCESSED_DEVICES] = devices or []
         # endregion
 
         _LOGGER.debug(self._log_formatter.format("exited"))
@@ -644,6 +647,7 @@ class Mesh:
 
         details = await self._async_gather_details(
             include_backhaul=True,
+            include_channel_scan=True,
             include_devices=True,
             include_firmware_update=True,
             include_firmware_update_settings=True,
@@ -694,6 +698,13 @@ class Mesh:
         self.__gather_details_executed = True  # pylint: disable=unused-private-member
         _LOGGER.debug(self._log_formatter.format("exited"))
 
+    async def async_get_channel_scan_info(self) -> Dict[str, Any]:
+        """Get the current state of the channel scan."""
+        resp = await self._async_gather_details(
+            include_channel_scan=True,
+        )
+        return resp.get(ATTR_CHANNEL_SCAN_INFO)
+
     async def async_get_device_from_id(
         self, device_id: str, force_refresh: bool = False
     ) -> Device | Node:
@@ -812,17 +823,16 @@ class Mesh:
         _LOGGER.debug(self._log_formatter.format("entered"))
 
         payload = {
-            **api.Defaults.PAYLOADS.get(api.Actions.GET_SPEEDTEST_RESULTS, {}),
+            **api.Defaults.payloads[api.Actions.GET_SPEEDTEST_RESULTS],
             "lastNumberOfResults": count,
         }
-        resp = await self._async_make_request(
+        _, resp = await self._async_make_request(
             action=api.Actions.GET_SPEEDTEST_RESULTS, payload=payload
         )
-        healthcheck_results = resp.get("healthCheckResults")
 
         _LOGGER.debug(self._log_formatter.format("exited"))
         return _process_speedtest_results(
-            speedtest_results=healthcheck_results,
+            speedtest_results=resp.data.get("healthCheckResults"),
             only_latest=only_latest,
             only_completed=only_completed,
         )
@@ -885,7 +895,7 @@ class Mesh:
         if not node_details:
             raise MeshDeviceNotFoundResponse
 
-        if node_details[0].type == NODE_TYPE_PRIMARY and not force:
+        if node_details[0].type == NodeType.PRIMARY and not force:
             raise MeshInvalidInput(f"{node_name} is a primary node. Use the force.")
 
         node_ip = [
@@ -952,6 +962,12 @@ class Mesh:
 
         _LOGGER.debug(self._log_formatter.format("exited"))
 
+    async def async_start_channel_scan(self) -> None:
+        """Start a channel scan on the mesh."""
+        _LOGGER.debug(self._log_formatter.format("entered"))
+        await self._async_make_request(action=api.Actions.START_CHANNEL_SCAN)
+        _LOGGER.debug(self._log_formatter.format("exited"))
+
     async def async_start_speedtest(self) -> None:
         """Instruct the mesh to carry out a Speedtest.
 
@@ -1099,6 +1115,14 @@ class Mesh:
             "isPaired", False
         )
 
+    @property
+    @needs_gather_details
+    def is_channel_scan_running(self) -> bool:
+        """Get the current state of channel scanning."""
+        return bool(
+            self._mesh_attributes.get(ATTR_CHANNEL_SCAN_INFO, {}).get("isRunning", None)
+        )
+
     @property
     @needs_gather_details
     def latest_speedtest_result(self) -> Optional[Dict]:
